<?php

namespace Drupal\file_to_s_three\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\Form\FormStateInterface;
/**
 * Provides a field type to upload file into S3 bucket.
 * 
 * @FieldType(
 *   id = "sthree",
 *   label = @Translation("Upload file to S3 Bucket"),
 *   description = @Translation("Field to upload any type of files into S3 Bucket."),
 *   default_formatter = "sthree_formatter",
 *   default_widget = "sthree_widget",
 * )
 */
 class UploadFieldType extends FieldItemBase {
	/**
	 * {@inheritdoc}
	 */
	public static function schema(FieldStorageDefinitionInterface $field_definition) {
	    return [
		// columns contains the values that the field will store
		'columns' => [
		  'bucketname' => [
			'type' => 'varchar',
			'length' => 256,
			'not null' => FALSE,
		  ],
		  'bucketregion' => [
			'type' => 'varchar',
			'length' => 256,
			'not null' => FALSE,
		  ],
		  'accesskey' => [
			'type' => 'varchar',
			'length' => 256,
			'not null' => FALSE,
		  ],
		  'secretkey' => [
			'type' => 'varchar',
			'length' => 256,
			'not null' => FALSE,
		  ],
		],
	  ];
	}

	/**
	 * {@inheritdoc}
	 */
	public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
	  $properties = [];
	  $properties['bucketname'] = DataDefinition::create('string')
		->setLabel(t('Bucket name'))
		->setRequired(TRUE);
	  $properties['bucketregion'] = DataDefinition::create('string')
		->setLabel(t('Bucket region'))
		->setRequired(TRUE);
	  $properties['accesskey'] = DataDefinition::create('string')
		->setLabel(t('Access key'))
		->setRequired(TRUE);
	  $properties['secretkey'] = DataDefinition::create('string')
		->setLabel(t('Secret key'))
		->setRequired(TRUE);

	  return $properties;
	}
	
	/**
	 * {@inheritdoc}
	 */
	public function isEmpty() {
	  $bucketname = $this->get('bucketname')->getValue();
	  return $bucketname === NULL || $bucketname === '';  
	}
	/**
	 * {@inheritdoc}
	 */
	public static function defaultFieldSettings() {
	  return [
		// Declare a single setting, with a default
		// value of 'large'
		'bucketname' => '',
		'bucketregion' => '',
		'accesskey' => '',
		'secretkey' => '',
	  ] + parent::defaultFieldSettings();
	}
	/**
	 * {@inheritdoc}
	 */
	public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
	$element = [];
	  // The key of the element should be the setting name
      $element['bucketname'] = [
		'#title' => $this->t('Bucket name'),
		'#type' => 'textfield',
		'#required' => TRUE,
		'#default_value' => $this->getSetting('bucketname'),
		'#description' => t('Please provide the unique S3 Buckect name.'),
		'#attributes' => array('placeholder' => t('Bucket name')),
	  ];
	  $element['bucketregion'] = [
		'#title' => $this->t('S3 Bucket Region'),
		'#type' => 'textfield',
		'#required' => TRUE,
		'#default_value' => $this->getSetting('bucketregion'),
		'#description' => t('Please provide the S3 Buckect region.'),
		'#attributes' => array('placeholder' => t('Bucket region')),
	  ];
	  $element['accesskey'] = [
		'#title' => $this->t('S3 Access Key'),
		'#type' => 'textfield',
		'#required' => TRUE,
		'#default_value' => $this->getSetting('accesskey'),
		'#description' => t('Please provide the S3 Buckect access key.'),
		'#attributes' => array('placeholder' => t('Access key value')),
	  ];
	 $element['secretkey'] = [
		'#title' => $this->t('S3 Secret Key'),
		'#type' => 'textfield',
		'#required' => TRUE,
		'#default_value' => $this->getSetting('secretkey'),
		'#description' => t('Please provide the S3 Buckect secret key.'),
		'#attributes' => array('placeholder' => t('Secret key value')),
	  ];
	  return $element;
	}
}