<?php

namespace Drupal\file_to_s_three\Plugin\Field\FieldWidget;

/**
 * A widget bar.
 *
 * @FieldWidget(
 *   id = "sthree_widget",
 *   label = @Translation("Upload widget"),
 *   field_types = {
 *     "sthree",
 *     "string"
 *   }
 * )
 */
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Aws\S3\S3Client;
use Drupal\file\Entity\File;

class UploadWidget extends WidgetBase {
  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $sthree_file_field = isset($items[$delta]->sthree_file_field) ? $items[$delta]->sthree_file_field : '';
    $element += [
      '#type' => 'managed_file',
      '#required' => TRUE,
	  '#upload_location' => 'public://',
	  '#multiple' => TRUE,
	  '#default_value' => $sthree_file_field,
	  '#description' => $this->getSetting('extensions'),
      '#element_validate' => [
        [static::class, 'validate'],
      ],
	  '#upload_validators' => array(
		'file_validate_extensions' => array($this->getSetting('extensions'),),
	  )
    ];
    return ['sthree_file_field' => $element];
  }
  
  /**
   * Validate the fields.
   */
  public static function validate($element, FormStateInterface $form_state) {
    
	$form_object = $form_state->getFormObject();
    $node = $form_object->getEntity();
	
	/* get entity type like node, taxonomy_term, taxonomy_vocabulary, ... */
	$entity_type = $node->getEntityTypeId();
	/* get machine name of content type */
    $bundle = $node->bundle(); 
	/* get machine name of file field */
	$field = substr($element['#name'], 0, -22);	
	
    $fieldConfig =  \Drupal::entityTypeManager()->getStorage('field_config')
    ->load($entity_type . '.' . $bundle . '.' . $field);

	try{
	  $formsubmitted = $form_state->getValues();
	  if ($formsubmitted['op'] == 'Save') {
	  /* Getting custom configuration parameters. */	  
      $bucketname = $fieldConfig->getSetting('bucketname');
	  $bucketregion = $fieldConfig->getSetting('bucketregion');
	  $accesskey = $fieldConfig->getSetting('accesskey');
	  $secretkey = $fieldConfig->getSetting('secretkey');
      /* Instantiate an Amazon S3 client. */
	  $s3Client = new S3Client([
        'version'     => 'latest',
	    'scheme' =>'http',
        'region'      => $bucketregion,
        'credentials' => [
          'key'    => $accesskey,
          'secret' => $secretkey,
        ],
      ]);
      foreach ($element['#value']['fids'] as $tempfileid) {
		$file = File::load($tempfileid);
        $filepath = $file->getFileUri();
        $filename= basename($filepath);
		if ($filepath !== '' && $filename !== '') {
		/* Moving file into S3 Bucket */	
        $result = $s3Client->putObject([
	      'Bucket' => $bucketname,
	      'Key'    => $filename,
	      'SourceFile' => $filepath,
	    ]);
		//\Drupal::messenger()->addMessage(t('File has been Uploaded succesfully '));
       }
	  }
	  }
	} catch(Exception $ex){
		\Drupal::logger('upload_to_s_three')->error($ex->getMessage());
	}
  }
 
  /**
   * {@inheritdoc}
  */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['extensions'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Extensions of the file'),
      '#default_value' => $this->getSetting('extensions'),
      '#required' => TRUE,
    ];
  return $element;
} 
  /**
 * {@inheritdoc}
 */
  public static function defaultSettings() {
    return [
	  // Create the custom setting 'extensions' and
	 // assign a default value if required
	 'extensions' => 'doc docx txt pdf png',
    ] + parent::defaultSettings();
  }
  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('Allowed file extenions to upload: @extensions', ['@extensions' => $this->getSetting('extensions')]);

    return $summary;
  }
}